# Generated by Django 3.2.13 on 2022-09-27 01:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_protocol_incident", "0014_auto_20220927_0400"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="protocolviolations",
            name="edc_protoco_id_b2c86f_idx",
        ),
        migrations.AddIndex(
            model_name="protocolviolations",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="edc_protoco_id_58f1f5_idx",
            ),
        ),
        migrations.AlterModelTable(
            name="actionsrequired",
            table=None,
        ),
        migrations.AlterModelTable(
            name="protocoldeviationviolation",
            table=None,
        ),
        migrations.AlterModelTable(
            name="protocolincident",
            table=None,
        ),
        migrations.AlterModelTable(
            name="protocolincidents",
            table=None,
        ),
        migrations.AlterModelTable(
            name="protocolviolations",
            table=None,
        ),
    ]
